#include <stdio.h>
#include <postgres.h>
#include <utils/palloc.h>

typedef struct {
  int2 area_code;
  int2 prefix;
  int2 number;
} us_phone_number;

us_phone_number	*us_phone_number_in(char *str);
char 		*us_phone_number_out(us_phone_number *in);

bool 		us_phone_number_lt(us_phone_number *a1, us_phone_number *a2);
bool 		us_phone_number_le(us_phone_number *a1, us_phone_number *a2);
bool 		us_phone_number_eq(us_phone_number *a1, us_phone_number *a2);
bool 		us_phone_number_ge(us_phone_number *a1, us_phone_number *a2);
bool 		us_phone_number_gt(us_phone_number *a1, us_phone_number *a2);
bool 		us_phone_number_ne(us_phone_number *a1, us_phone_number *a2);
int2 		us_phone_number_cmp(us_phone_number *a1, us_phone_number *a2);

us_phone_number *us_phone_number_in(char *str) {
  int area_code, prefix, number;
  char packed_number[16];
  us_phone_number *result;
  int i,j;

  i = 0;
  for (j=0; j<strlen(str); j++)
    if (strchr("0123456789", str[j]))
      packed_number[i++] = str[j];
  packed_number[i] = 0; /* make sure it's null terminated */

  if (strlen(packed_number) == 10) {
      sscanf(packed_number, "%3d%3d%4d", &area_code, &prefix, &number);
  } else {
    if (strlen(packed_number) == 7) {
      area_code = 0; 
      sscanf(packed_number, "%3d%4d", &prefix, &number);
    } else {
      area_code = 0; prefix = 0; number = 0;
    } 
  }

  if ((area_code > 999) || (area_code < 0)) area_code = 0;
  if ((prefix > 999) || (prefix < 0)) prefix = 0;
  if ((area_code > 9999) || (area_code < 0)) number = 0;
  
  result = (us_phone_number *)palloc(sizeof(us_phone_number));
  result->area_code = area_code;
  result->prefix = prefix;
  result->number = number;

  return (result);

}

char *us_phone_number_out(us_phone_number *number) {
  char *result;
  if (number == NULL)
    return (NULL);
  result = (char *)palloc(15);
  if (result != NULL)
    sprintf(result, "(%03d) %03d-%04d", 
                    number->area_code, number->prefix, number->number);

  return (result);
}

bool us_phone_number_lt(us_phone_number *a1, us_phone_number *a2) {
  if ((a1->area_code != 0) && (a2->area_code != 0)) {
    if (a1->area_code == a2->area_code) {
      if (a1->prefix == a2->prefix) {
        if (a1->number < a2->number) return (true);
      } else {
        if (a1->prefix < a2->prefix) return (true);
      }
    } else {
      if (a1->area_code < a2->area_code) return(true);
    }
  } else {
    if (a1->prefix == a2->prefix) {
      if (a1->number < a2->number) return (true);
    } else {
      if (a1->prefix < a2->prefix) return (true);
    }
  }
  return (false);
}

bool us_phone_number_le(us_phone_number *a1, us_phone_number *a2) {
  if ((a1->area_code != 0) && (a2->area_code != 0)) {
    if (a1->area_code == a2->area_code) {
      if (a1->prefix == a2->prefix) {
        if (a1->number < a2->number) return (true);
      } else {
        if (a1->prefix <= a2->prefix) return (true);
      }
    } else {
      if (a1->area_code < a2->area_code) return(true);
    }
  } else {
    if (a1->prefix == a2->prefix) {
      if (a1->number <= a2->number) return (true);
    } else {
      if (a1->prefix < a2->prefix) return (true);
    }
  }
  return (false);
}

bool us_phone_number_eq(us_phone_number *a1, us_phone_number *a2) {
 return ((a1->prefix == a2->prefix) &&
         (a1->number == a2->number) &&
         ( (a1->area_code == a2->area_code) ||
           (a1->area_code == 0) || 
           (a2->area_code == 0)   ));
}

bool us_phone_number_ge(us_phone_number *a1, us_phone_number *a2) {
  if ((a1->area_code != 0) && (a2->area_code != 0)) {
    if (a1->area_code == a2->area_code) {
      if (a1->prefix == a2->prefix) {
        if (a1->number > a2->number) return (true);
      } else {
        if (a1->prefix >= a2->prefix) return (true);
      }
    } else {
      if (a1->area_code > a2->area_code) return(true);
    }
  } else {
    if (a1->prefix == a2->prefix) {
      if (a1->number >= a2->number) return (true);
    } else {
      if (a1->prefix > a2->prefix) return (true);
    }
  }
  return (false);
}

bool us_phone_number_gt(us_phone_number *a1, us_phone_number *a2) {
  if ((a1->area_code != 0) && (a2->area_code != 0)) {
    if (a1->area_code == a2->area_code) {
      if (a1->prefix == a2->prefix) {
        if (a1->number > a2->number) return (true);
      } else {
        if (a1->prefix > a2->prefix) return (true);
      }
    } else {
      if (a1->area_code > a2->area_code) return(true);
    }
  } else {
    if (a1->prefix == a2->prefix) {
      if (a1->number > a2->number) return (true);
    } else {
      if (a1->prefix > a2->prefix) return (true);
    }
  }
  return (false);
}

bool us_phone_number_ne(us_phone_number *a1, us_phone_number *a2) {
  if ((a1->area_code == 0) || (a2->area_code == 0)) {
    return ((a1->prefix    != a2->prefix    ) ||
            (a1->number    != a2->number    ) );    
  } else {
    return ((a1->prefix    != a2->prefix    ) ||
            (a1->number    != a2->number    ) ||
            (a1->area_code != a2->area_code ));
  }
}

int2 us_phone_number_cmp(us_phone_number *a1, us_phone_number *a2) {
  if (a1->prefix < a2->prefix) return (-1);
  if (a1->prefix > a2->prefix) return (1);
  if (a1->number < a2->number) return (-1);
  if (a1->number > a2->number) return (1);
  if ((a1->area_code == 0) || (a2->area_code == 0)) return (0);
  if (a1->area_code < a2->area_code) return (-1);
  if (a1->area_code > a2->area_code) return (1);
  return (0);
}
